using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Parser;

namespace sharkedit
{
    public partial class Form1 : Form
    {
        string name;
        SharkFile shark;

        public Form1()
        {
            name = "";
            shark = null;
            InitializeComponent();
        }

        private void loadTree(TreeNodeCollection treeNodes, SNode sharkNode)
        {
            foreach(SNode anode in sharkNode.subnodes)
            {
                TreeNode nnode = treeNodes.Add(anode.name);
                nnode.Tag = anode;
                switch (anode.type)
                {                        
                    case SharkType.ArraySub:
                        loadTree(nnode.Nodes, anode);
                        break;
                    case SharkType.Sub:
                        if (anode["type"] != null)
                            nnode.Text += " [" + (string)(anode["type"]) + "]";
                        loadTree(nnode.Nodes, anode);
                        break;           
                }
            }                        
        }

        private void btLoad_Click(object sender, EventArgs e)
        {            
            OpenFileDialog dlg = new OpenFileDialog();         
            dlg.Filter = "All files (*.*)|*.*";
            dlg.FilterIndex = 1;
            
            if(dlg.ShowDialog() == DialogResult.OK)
            {
                try                   
                {
                    tree.BeginUpdate();
                    tree.Nodes.Clear();
                    shark = new SharkFile(dlg.FileName);
                    loadTree(tree.Nodes, shark.root);
                    tree.EndUpdate();
                    name = dlg.FileName;
                    this.Text = "SharkEdit " + name;
                }
                catch (Exception) {
                    tree.Nodes.Clear();                    
                    MessageBox.Show("Error reading file !"); 
                }
            }
        }

        private void btSave_Click(object sender, EventArgs e)
        {
            if (name == "")
            {
                MessageBox.Show("No load file loaded");
                return;
            }
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "All files (*.*)|*.*";
            dlg.FilterIndex = 1;
            dlg.FileName = name;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    shark.filename = dlg.FileName;
                    shark.save();
                    name = dlg.FileName;
                }
                catch (Exception)
                {
                    MessageBox.Show("Error writing file !");
                }
            }
        }

        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            list.Rows.Clear();
            list.AllowUserToAddRows = false;
            list.AllowUserToDeleteRows = false;
            if (tree.SelectedNode != null)
            {
                SNode node = (SNode)tree.SelectedNode.Tag;                
                switch (node.type)
                {
                    case SharkType.Int:
                        list.Rows.Add((long)node.reference);
                        list.Columns[0].ValueType = typeof(long);
                        break;
                    case SharkType.String:
                        list.Rows.Add((string)node.reference);
                        list.Columns[0].ValueType = typeof(string);
                        break;
                    case SharkType.Float:
                        list.Rows.Add(Convert.ToString((float)node.reference));
                        list.Columns[0].ValueType = typeof(float);
                        break;
                    case SharkType.ArrayInt:
                        for (int i = 0; i < node.count; i++)
                            list.Rows.Add(((long[])node.reference)[i]);
                        //list.AllowUserToAddRows = true;
                        //list.AllowUserToDeleteRows = true;
                        list.Columns[0].ValueType = typeof(long);
                        break;                        
                    case SharkType.ArrayFloat:
                        for (int i = 0; i < node.count; i++)
                            list.Rows.Add(((float[])node.reference)[i]);
                        //list.AllowUserToDeleteRows = true;
                        //list.AllowUserToAddRows = true;
                        list.Columns[0].ValueType = typeof(float);
                        break;
                    case SharkType.ArrayString:
                        for (int i = 0; i < node.count; i++)
                            list.Rows.Add(((string[])node.reference)[i]);
                        //list.AllowUserToDeleteRows = true;
                        //list.AllowUserToAddRows = true;
                        list.Columns[0].ValueType = typeof(string);                        
                        break;
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            list.Rows.Clear();
        }

        private void list_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            if (tree.SelectedNode != null && list.SelectedCells[0].Value != null && list.SelectedCells[0].Value.ToString() != "")
            {
                SNode node = (SNode)tree.SelectedNode.Tag;
                switch (node.type)
                {
                    case SharkType.Int:
                        node.reference = (long)list.SelectedCells[0].Value;                        
                        break;
                    case SharkType.String:
                        node.reference = (string)list.SelectedCells[0].Value;
                        break;
                    case SharkType.Float:
                        node.reference = (float)list.SelectedCells[0].Value;
                        break;
                    case SharkType.ArrayInt:
                        {
                            long[] l = new long[list.Rows.Count];
                            for (int i = 0; i < l.Length; i++)
                                l[i] = (long)(list.Rows[i].Cells[0].Value);
                            node.reference = l;
                        } break;
                    case SharkType.ArrayFloat:
                        {
                            float[] l = new float[list.Rows.Count];
                            for (int i = 0; i < l.Length; i++)
                                l[i] = (float)(list.Rows[i].Cells[0].Value);
                            node.reference = l;
                        } break;
                    case SharkType.ArrayString:
                        {
                            string[] l = new string[list.Rows.Count];
                            for (int i = 0; i < l.Length; i++)
                                l[i] = (string)(list.Rows[i].Cells[0].Value);
                            node.reference = l;
                        } break;
                }
            }
        }

        private void list_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            MessageBox.Show(this, "wrong data format", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void btInject_Click(object sender, EventArgs e)
        {
            if (name == "")            
                MessageBox.Show("No load file loaded");                            
            else
                Pak.InjectManager.instance.inject(name);
        }

        private void btFind_Click(object sender, EventArgs e)
        {
            TreeNode node = tree.SelectedNode;
            if (node == null) node = tree.Nodes[0];

            while (node != null)
            {
                // next node
                if (node.FirstNode != null)
                    node = node.FirstNode;
                else 
                {
                    while (node.NextNode == null)
                    {
                        if (node.Parent == null) return;
                        node = node.Parent;
                    }
                    node = node.NextNode;
                }
                SNode snode = (SNode)node.Tag;
                if (snode.type == SharkType.String && ((string)(snode.reference)).Contains(txtFind.Text))
                {
                    tree.SelectedNode = node;
                    tree.Select();
                    break;
                }                
            }
        }
    }

}